@@ -25,7 +25,10 @@
 and attributes of Arch/BIM objects.
 """
 
-import FreeCAD,json
+import json
+
+import FreeCAD
+import ArchIFCSchema
 
 if FreeCAD.GuiUp:
     from PySide.QtCore import QT_TRANSLATE_NOOP
@@ -33,8 +36,6 @@ else:
     def QT_TRANSLATE_NOOP(ctx,txt):
         return txt
 
-import ArchIFCSchema
-
 def uncamel(t):
     return ''.join(map(lambda x: x if x.islower() else " "+x, t[3:]))[1:]
 
@@ -236,7 +237,7 @@ class IfcRoot:
         Do so using the .addIfcAttributeValueExpressions() method.
 
         Learn more about expressions here:
-        https://wiki.freecadweb.org/Expressions
+        https://wiki.freecad.org/Expressions
 
         Do not add the attribute if the object has a property with the
         attribute's name. Also do not add the attribute if its name is
@@ -314,7 +315,7 @@ class IfcRoot:
         - LongName
 
         Learn more about expressions here:
-        https://wiki.freecadweb.org/Expressions
+        https://wiki.freecad.org/Expressions
 
         Parameters
         ----------
