@@ -384,10 +384,10 @@ class PathDressupTagViewProvider:
         #    return True
         return False
 
-    def __getstate__(self):
+    def dumps(self):
         return None
 
-    def __setstate__(self, state):
+    def loads(self, state):
         return None
 
     def setupColors(self):
