@@ -46,7 +46,6 @@ class SpreadsheetCases(unittest.TestCase):
     def testAggregates(self):
         """ Test all aggregate functions """
         sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
-        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
         sheet.set('B13',  '4')
         sheet.set('B14',  '5')
         sheet.set('B15',  '6')
@@ -165,7 +164,6 @@ class SpreadsheetCases(unittest.TestCase):
 
     def testFunctions(self):
         """ Test all built-in simple functions """
-        doc = FreeCAD.newDocument()
         sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
         sheet.set('A1',  '=cos(60)')   # Cos
         sheet.set('B1',  '=cos(60deg)')
@@ -366,7 +364,6 @@ class SpreadsheetCases(unittest.TestCase):
         self.assertMostlyEqual(sheet.B27, l)
         self.assertTrue(sheet.C27.startswith(u'ERR: Units must be equal'))
         self.assertMostlyEqual(sheet.D27, Units.Quantity("3 mm"))
-        FreeCAD.closeDocument(doc.Name)
 
     def testRelationalOperators(self):
         """ Test relational operators """
@@ -894,7 +891,7 @@ class SpreadsheetCases(unittest.TestCase):
         pla = FreeCAD.Placement(vec,rot)
         ipla = pla.inverse()
 
-        sheet.set('A1', '=create(<<vector>>, 2, 1, 2)')
+        sheet.set('A1', '=vector(2, 1, 2)')
 
         # different ways of calling mscale()
         sheet.set('B1', '=mscale(create(<<matrix>>), A1)')
@@ -907,10 +904,10 @@ class SpreadsheetCases(unittest.TestCase):
         sheet.set('D2', '=A2^0')
         sheet.set('E2', '=A2^1')
         sheet.set('F2', '=A2^2')
-        sheet.set('G2', '=create(<<matrix>>, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)')
+        sheet.set('G2', '=matrix(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)')
         sheet.set('H2', '=G2^-1')
 
-        sheet.set('A3', '=create(<<rotation>>, create(<<vector>>, 0, 1, 0), 45)')
+        sheet.set('A3', '=rotation(vector(0, 1, 0), 45)')
 
         # test rotation power operation
         sheet.set('B3', '=A3^-2')
@@ -919,7 +916,7 @@ class SpreadsheetCases(unittest.TestCase):
         sheet.set('E3', '=A3^1')
         sheet.set('F3', '=A3^2')
 
-        sheet.set('A4', '=create(<<placement>>, A1, A3)')
+        sheet.set('A4', '=placement(A1, A3)')
 
         # test placement power operation
         sheet.set('B4', '=A4^-2')
@@ -944,6 +941,20 @@ class SpreadsheetCases(unittest.TestCase):
         sheet.set('E6', '=(E2 * E3)^-1 * E4^-1 * E5')
         sheet.set('F6', '=(F3*F4*F2)^-1 * F5')
 
+        # Rotate and translate.
+        sheet.set('A7', '=placement(vector(1; 2; 3), vector(1; 0; 0); 0)')
+        sheet.set('B7', '=mrotate(A7; vector(1; 0; 0); 90)')
+        sheet.set('C7', '=mrotatex(A7; 90)')
+        sheet.set('D7', '=mrotatey(A7; 90)')
+        sheet.set('E7', '=mrotatez(A7; 90)')
+        sheet.set('F7', '=mtranslate(A7; vector(1; 2; 3))')
+        sheet.set('G7', '=mtranslate(A7; 1; 2; 3)')
+
+        # Compatibility with old syntax.
+        sheet.set('A8', '=create(<<vector>>, 2, 1, 2)')
+        sheet.set('B8', '=create(<<rotation>>, create(<<vector>>, 0, 1, 0), 45)')
+        sheet.set('C8', '=create(<<placement>>, A8, B8)')
+
         self.doc.recompute()
 
         self.assertEqual(sheet.A1,vec)
@@ -1014,6 +1025,18 @@ class SpreadsheetCases(unittest.TestCase):
         self.assertLess(sheet.E6.distanceToPoint(vec),tol)
         self.assertLess(sheet.F6.distanceToPoint(vec),tol)
 
+        self.assertTrue(sheet.A7.Base.isEqual(FreeCAD.Vector(1, 2, 3), tol))
+        self.assertTrue(sheet.B7.Base.isEqual(FreeCAD.Vector(1, -3, 2), tol))
+        self.assertTrue(sheet.C7.Base.isEqual(FreeCAD.Vector(1, -3, 2), tol))
+        self.assertTrue(sheet.D7.Base.isEqual(FreeCAD.Vector(3, 2.0, -1), tol))
+        self.assertTrue(sheet.E7.Base.isEqual(FreeCAD.Vector(-2, 1, 3.0), tol))
+        self.assertTrue(sheet.F7.Base.isEqual(FreeCAD.Vector(2, 4, 6), tol))
+        self.assertTrue(sheet.G7.Base.isEqual(FreeCAD.Vector(2, 4, 6), tol))
+
+        self.assertEqual(sheet.A8, vec)
+        self.assertEqual(sheet.B8, rot)
+        self.assertEqual(sheet.C8, pla)
+
     def testIssue3128(self):
         """ Regression test for issue 3128; mod should work with arbitrary units for both arguments """
         sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
@@ -1059,7 +1082,7 @@ class SpreadsheetCases(unittest.TestCase):
         self.doc.recompute()
         sheet.set('A3', '')
         sheet.set('A3', 'A1')
-        self.assertEqual(sheet.getContents('A3'), 'A1')
+        self.assertEqual(sheet.getContents('A3'), '\'A1')
 
     def testInsertRowsAlias(self):
         """ Regression test for issue 4429; insert rows to sheet with aliases"""
@@ -1144,7 +1167,7 @@ class SpreadsheetCases(unittest.TestCase):
         sheet.setAlias('C3','test')
 
     def testCrossLinkEmptyPropertyName(self):
-        # https://forum.freecadweb.org/viewtopic.php?f=3&t=58603
+        # https://forum.freecad.org/viewtopic.php?f=3&t=58603
         base = FreeCAD.newDocument("base")
         sheet = base.addObject('Spreadsheet::Sheet','Spreadsheet')
         sheet.setAlias('A1', 'x')
@@ -1185,7 +1208,7 @@ class SpreadsheetCases(unittest.TestCase):
         FreeCAD.closeDocument(base.Name)
 
     def testExpressionWithAlias(self):
-        # https://forum.freecadweb.org/viewtopic.php?p=564502#p564502
+        # https://forum.freecad.org/viewtopic.php?p=564502#p564502
         ss1 = self.doc.addObject("Spreadsheet::Sheet", "Input")
         ss1.setAlias('A1', 'one')
         ss1.setAlias('A2', 'two')
@@ -1345,6 +1368,101 @@ class SpreadsheetCases(unittest.TestCase):
         with self.assertRaises(AttributeError):
             self.assertEqual(ss1.B1, "fail")
 
+    def testGetUsedCells(self):
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        test_cells = ['B13','C14','D15']
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,str(i))
+
+        used_cells = sheet.getUsedCells()
+        self.assertEqual(len(used_cells), len(test_cells))
+        for cell in test_cells:
+            self.assertTrue(cell in used_cells)
+
+        for cell in test_cells:
+            sheet.set(cell,"")
+            sheet.setAlignment(cell,"center")
+        non_empty_cells = sheet.getUsedCells()
+        self.assertEqual(len(non_empty_cells), len(test_cells)) # Alignment counts as "used"
+
+    def testGetUsedRange(self):
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        test_cells = ['C5','Z3','D10','E20']
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,str(i))
+        used_range = sheet.getUsedRange()
+        self.assertEquals(used_range,("C3","Z20"))
+
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,"")
+            sheet.setAlignment(cell,"center")
+        used_range = sheet.getUsedRange()
+        self.assertEquals(used_range,("C3","Z20"))
+
+    def testGetNonEmptyCells(self):
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        test_cells = ['B13','C14','D15']
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,str(i))
+
+        non_empty_cells = sheet.getNonEmptyCells()
+        self.assertEqual(len(non_empty_cells), len(test_cells))
+        for cell in test_cells:
+            self.assertTrue(cell in non_empty_cells)
+
+        for cell in test_cells:
+            sheet.set(cell,"")
+            sheet.setAlignment(cell,"center")
+        non_empty_cells = sheet.getNonEmptyCells()
+        self.assertEqual(len(non_empty_cells), 0) # Alignment does not count as "non-empty"
+
+    def testGetNonEmptyRange(self):
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        test_cells = ['C5','Z3','D10','E20']
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,str(i))
+        non_empty_range = sheet.getNonEmptyRange()
+        self.assertEquals(non_empty_range,("C3","Z20"))
+
+        for i,cell in enumerate(test_cells):
+            sheet.set(cell,"")
+            sheet.setAlignment(cell,"center")
+        more_cells = ['D10','X5','E10','K15']
+        for i,cell in enumerate(more_cells):
+            sheet.set(cell,str(i))
+        non_empty_range = sheet.getNonEmptyRange()
+        self.assertEquals(non_empty_range,("D5","X15"))
+
+    def testAliasEmptyCell(self):
+        # https://github.com/FreeCAD/FreeCAD/issues/7841
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        sheet.setAlias('A1', 'aliasOfEmptyCell')
+        self.assertEqual(sheet.getCellFromAlias("aliasOfEmptyCell"),"A1")
+
+    def testParensAroundCondition(self):
+        """ Parens around a condition should be accepted """
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+
+        sheet.set('A1', '=(1 == 1) ? 1 : 0')
+        self.doc.recompute()
+        self.assertEqual(sheet.getContents('A1'), '=1 == 1 ? 1 : 0')
+        self.assertEqual(sheet.A1, 1)
+
+    def testIssue6395(self):
+        """ Testing strings are correctly saved and restored"""
+        sheet = self.doc.addObject('Spreadsheet::Sheet','Spreadsheet')
+        sheet.set('A1', '\'36C') # Use a string that may be parsed as a quantity
+        self.doc.recompute()
+
+        self.doc.saveAs(self.TempPath + os.sep + 'string.fcstd')
+        FreeCAD.closeDocument(self.doc.Name)
+
+        self.doc = FreeCAD.openDocument(self.TempPath + os.sep + 'string.fcstd')
+
+        sheet = self.doc.getObject('Spreadsheet')
+        self.assertEqual(sheet.getContents('A1'), '\'36C')
+        self.assertEqual(sheet.get('A1'), '36C')
+
     def tearDown(self):
         #closing doc
         FreeCAD.closeDocument(self.doc.Name)
